@@ -89,10 +89,10 @@ G_DEFINE_TYPE_WITH_CODE (GtkAppChooserButton, gtk_app_chooser_button, GTK_TYPE_C
 struct _GtkAppChooserButtonPrivate {
   GtkListStore *store;
 
-  int last_active;
   gchar *content_type;
-  gboolean show_dialog_item;
   gchar *heading;
+  gint last_active;
+  gboolean show_dialog_item;
 
   GHashTable *custom_item_names;
 };
@@ -153,25 +153,29 @@ select_application_func_cb (GtkTreeModel *model,
   SelectAppData *data = user_data;
   GAppInfo *app_to_match = data->info, *app = NULL;
   gboolean custom;
+  gboolean result;
 
   gtk_tree_model_get (model, iter,
                       COLUMN_APP_INFO, &app,
                       COLUMN_CUSTOM, &custom,
                       -1);
 
-  /* cutsom items are always after GAppInfos, so iterating further here
+  /* custom items are always after GAppInfos, so iterating further here
    * is just useless.
    */
   if (custom)
-    return TRUE;
-
-  if (g_app_info_equal (app, app_to_match))
+    result = TRUE;
+  else if (g_app_info_equal (app, app_to_match))
     {
       gtk_combo_box_set_active_iter (GTK_COMBO_BOX (data->self), iter);
-      return TRUE;
+      result = TRUE;
     }
+  else
+    result = FALSE;
 
-  return FALSE;
+  g_object_unref (app);
+
+  return result;
 }
 
 static void
@@ -210,6 +214,8 @@ other_application_dialog_response_cb (GtkDialog *dialog,
 
   info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (dialog));
 
+  gtk_widget_destroy (GTK_WIDGET (dialog));
+
   /* refresh the combobox to get the new application */
   gtk_app_chooser_refresh (GTK_APP_CHOOSER (self));
   gtk_app_chooser_button_select_application (self, info);
@@ -248,7 +254,7 @@ gtk_app_chooser_button_ensure_dialog_item (GtkAppChooserButton *self,
 {
   GtkTreeIter iter, iter2;
 
-  if (!self->priv->show_dialog_item)
+  if (!self->priv->show_dialog_item || !self->priv->content_type)
     return;
 
   if (prev_iter == NULL)
@@ -275,7 +281,8 @@ gtk_app_chooser_button_populate (GtkAppChooserButton *self)
   gboolean cycled_recommended;
 
 #ifndef G_OS_WIN32
-  recommended_apps = g_app_info_get_recommended_for_type (self->priv->content_type);
+  if (self->priv->content_type)
+    recommended_apps = g_app_info_get_recommended_for_type (self->priv->content_type);
 #endif
   cycled_recommended = FALSE;
 
@@ -325,14 +332,6 @@ gtk_app_chooser_button_build_ui (GtkAppChooserButton *self)
   GtkCellRenderer *cell;
   GtkCellArea *area;
 
-  self->priv->store = gtk_list_store_new (NUM_COLUMNS,
-                                          G_TYPE_APP_INFO,
-                                          G_TYPE_STRING, /* name */
-                                          G_TYPE_STRING, /* label */
-                                          G_TYPE_ICON,
-                                          G_TYPE_BOOLEAN, /* separator */
-                                          G_TYPE_BOOLEAN); /* custom */
-
   gtk_combo_box_set_model (GTK_COMBO_BOX (self),
                            GTK_TREE_MODEL (self->priv->store));
 
@@ -457,8 +456,6 @@ gtk_app_chooser_button_constructed (GObject *obj)
   if (G_OBJECT_CLASS (gtk_app_chooser_button_parent_class)->constructed != NULL)
     G_OBJECT_CLASS (gtk_app_chooser_button_parent_class)->constructed (obj);
 
-  g_assert (self->priv->content_type != NULL);
-
   gtk_app_chooser_button_build_ui (self);
 }
 
@@ -521,6 +518,8 @@ gtk_app_chooser_button_finalize (GObject *obj)
   g_free (self->priv->content_type);
   g_free (self->priv->heading);
 
+  g_object_unref (self->priv->store);
+
   G_OBJECT_CLASS (gtk_app_chooser_button_parent_class)->finalize (obj);
 }
 
@@ -605,6 +604,14 @@ gtk_app_chooser_button_init (GtkAppChooserButton *self)
   self->priv->custom_item_names =
     g_hash_table_new_full (g_str_hash, g_str_equal,
                            g_free, NULL);
+
+  self->priv->store = gtk_list_store_new (NUM_COLUMNS,
+                                          G_TYPE_APP_INFO,
+                                          G_TYPE_STRING, /* name */
+                                          G_TYPE_STRING, /* label */
+                                          G_TYPE_ICON,
+                                          G_TYPE_BOOLEAN, /* separator */
+                                          G_TYPE_BOOLEAN); /* custom */
 }
 
 static gboolean
@@ -753,14 +760,15 @@ gtk_app_chooser_button_append_custom_item (GtkAppChooserButton *self,
 }
 
 /**
- * gtk_app_chooser_button_select_custom_item:
+ * gtk_app_chooser_button_set_active_custom_item:
  * @self: a #GtkAppChooserButton
  * @name: the name of the custom item
  *
  * Selects a custom item previously added with
  * gtk_app_chooser_button_append_custom_item().
- * Use gtk_app_chooser_refresh() to bring the selection to its initial
- * state.
+ *
+ * Use gtk_app_chooser_refresh() to bring the selection
+ * to its initial state.
  *
  * Since: 3.0
  */
