@@ -31,9 +31,162 @@
 #include "gtkbuilderprivate.h"
 
 
+/**
+ * SECTION:gtkliststore
+ * @Short_description: A list-like data structure that can be used with the GtkTreeView
+ * @Title: GtkListStore
+ * @See_also:#GtkTreeModel, #GtkTreeStore
+ *
+ * The #GtkListStore object is a list model for use with a #GtkTreeView
+ * widget.  It implements the #GtkTreeModel interface, and consequentialy,
+ * can use all of the methods available there.  It also implements the
+ * #GtkTreeSortable interface so it can be sorted by the view.
+ * Finally, it also implements the tree <link linkend="gtktreednd">drag and
+ * drop</link> interfaces.
+ *
+ * The #GtkListStore can accept most GObject types as a column type, though
+ * it can't accept all custom types.  Internally, it will keep a copy of
+ * data passed in (such as a string or a boxed pointer).  Columns that
+ * accept #GObject<!-- -->s are handled a little differently.  The
+ * #GtkListStore will keep a reference to the object instead of copying the
+ * value.  As a result, if the object is modified, it is up to the
+ * application writer to call gtk_tree_model_row_changed() to emit the
+ * #GtkTreeModel::row_changed signal.  This most commonly affects lists with
+ * #GdkPixbuf<!-- -->s stored.
+ *
+ * <example>
+ * <title>Creating a simple list store.</title>
+ * <programlisting>
+ * enum {
+ *   COLUMN_STRING,
+ *   COLUMN_INT,
+ *   COLUMN_BOOLEAN,
+ *   N_COLUMNS
+ * };
+ *
+ * {
+ *   GtkListStore *list_store;
+ *   GtkTreePath *path;
+ *   GtkTreeIter iter;
+ *   gint i;
+ *
+ *   list_store = gtk_list_store_new (N_COLUMNS,
+ *                                    G_TYPE_STRING,
+ *                                    G_TYPE_INT,
+ *                                    G_TYPE_BOOLEAN);
+ *
+ *   for (i = 0; i < 10; i++)
+ *     {
+ *       gchar *some_data;
+ *
+ *       some_data = get_some_data (i);
+ *
+ *       // Add a new row to the model
+ *       gtk_list_store_append (list_store, &iter);
+ *       gtk_list_store_set (list_store, &iter,
+ *                           COLUMN_STRING, some_data,
+ *                           COLUMN_INT, i,
+ *                           COLUMN_BOOLEAN,  FALSE,
+ *                           -1);
+ *
+ *       /<!---->* As the store will keep a copy of the string internally, we
+ *        * free some_data.
+ *        *<!---->/
+ *       g_free (some_data);
+ *     }
+ *
+ *   // Modify a particular row
+ *   path = gtk_tree_path_new_from_string ("4");
+ *   gtk_tree_model_get_iter (GTK_TREE_MODEL (list_store),
+ *                            &iter,
+ *                            path);
+ *   gtk_tree_path_free (path);
+ *   gtk_list_store_set (list_store, &iter,
+ *                       COLUMN_BOOLEAN, TRUE,
+ *                       -1);
+ * }
+ * </programlisting>
+ * </example>
+ *
+ * <refsect2>
+ * <title>Performance Considerations</title>
+ * Internally, the #GtkListStore was implemented with a linked list with a
+ * tail pointer prior to GTK+ 2.6.  As a result, it was fast at data
+ * insertion and deletion, and not fast at random data access.  The
+ * #GtkListStore sets the #GTK_TREE_MODEL_ITERS_PERSIST flag, which means
+ * that #GtkTreeIter<!-- -->s can be cached while the row exists.  Thus, if
+ * access to a particular row is needed often and your code is expected to
+ * run on older versions of GTK+, it is worth keeping the iter around.
+ * </refsect2>
+ * <refsect2>
+ * <title>Atomic Operations</title>
+ * It is important to note that only the methods
+ * gtk_list_store_insert_with_values() and gtk_list_store_insert_with_valuesv()
+ * are atomic, in the sense that the row is being appended to the store and the
+ * values filled in in a single operation with regard to #GtkTreeModel signaling.
+ * In contrast, using e.g. gtk_list_store_append() and then gtk_list_store_set()
+ * will first create a row, which triggers the #GtkTreeModel::row-inserted signal
+ * on #GtkListStore. The row, however, is still empty, and any signal handler
+ * connecting to #GtkTreeModel::row-inserted on this particular store should be prepared
+ * for the situation that the row might be empty. This is especially important
+ * if you are wrapping the #GtkListStore inside a #GtkTreeModelFilter and are
+ * using a #GtkTreeModelFilterVisibleFunc. Using any of the non-atomic operations
+ * to append rows to the #GtkListStore will cause the
+ * #GtkTreeModelFilterVisibleFunc to be visited with an empty row first; the
+ * function must be prepared for that.
+ * </refsect2>
+ * <refsect2 id="GtkListStore-BUILDER-UI">
+ * <title>GtkListStore as GtkBuildable</title>
+ * <para>
+ * The GtkListStore implementation of the GtkBuildable interface allows
+ * to specify the model columns with a &lt;columns&gt; element that may
+ * contain multiple &lt;column&gt; elements, each specifying one model
+ * column. The "type" attribute specifies the data type for the column.
+ *
+ * Additionally, it is possible to specify content for the list store
+ * in the UI definition, with the &lt;data&gt; element. It can contain
+ * multiple &lt;row&gt; elements, each specifying to content for one
+ * row of the list model. Inside a &lt;row&gt;, the &lt;col&gt; elements
+ * specify the content for individual cells.
+ *
+ * Note that it is probably more common to define your models
+ * in the code, and one might consider it a layering violation
+ * to specify the content of a list store in a UI definition,
+ * <emphasis>data</emphasis>, not <emphasis>presentation</emphasis>,
+ * and common wisdom is to separate the two, as far as possible.
+ * <!-- FIXME a bit inconclusive -->
+ *
+ * <example>
+ * <title>A UI Definition fragment for a list store</title>
+ * <programlisting><![CDATA[
+ * <object class="GtkListStore">
+ *   <columns>
+ *     <column type="gchararray"/>
+ *     <column type="gchararray"/>
+ *     <column type="gint"/>
+ *   </columns>
+ *   <data>
+ *     <row>
+ *       <col id="0">John</col>
+ *       <col id="1">Doe</col>
+ *       <col id="2">25</col>
+ *     </row>
+ *     <row>
+ *       <col id="0">Johan</col>
+ *       <col id="1">Dahlin</col>
+ *       <col id="2">50</col>
+ *     </row>
+ *   </data>
+ * </object>
+ * ]]></programlisting>
+ * </example>
+ * </para>
+ * </refsect2>
+ */
+
+
 struct _GtkListStorePrivate
 {
-  GtkSortType order;
   GtkTreeIterCompareFunc default_sort_func;
 
   GDestroyNotify default_sort_destroy;
@@ -45,6 +198,8 @@ struct _GtkListStorePrivate
   gint sort_column_id;
   gint length;
 
+  GtkSortType order;
+
   guint columns_dirty : 1;
 
   gpointer default_sort_data;
@@ -52,8 +207,6 @@ struct _GtkListStorePrivate
 };
 
 #define GTK_LIST_STORE_IS_SORTED(list) (((GtkListStore*)(list))->priv->sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
-#define VALID_ITER(iter, list_store) ((iter)!= NULL && (iter)->user_data != NULL && list_store->priv->stamp == (iter)->stamp && !g_sequence_iter_is_end ((iter)->user_data) && g_sequence_iter_get_sequence ((iter)->user_data) == list_store->priv->seq)
-
 static void         gtk_list_store_tree_model_init (GtkTreeModelIface *iface);
 static void         gtk_list_store_drag_source_init(GtkTreeDragSourceIface *iface);
 static void         gtk_list_store_drag_dest_init  (GtkTreeDragDestIface   *iface);
@@ -247,6 +400,17 @@ gtk_list_store_init (GtkListStore *list_store)
   priv->length = 0;
 }
 
+static gboolean
+iter_is_valid (GtkTreeIter  *iter,
+               GtkListStore *list_store)
+{
+  return iter != NULL && 
+         iter->user_data != NULL &&
+         list_store->priv->stamp == iter->stamp &&
+         !g_sequence_iter_is_end (iter->user_data) &&
+         g_sequence_iter_get_sequence (iter->user_data) == list_store->priv->seq;
+}
+
 /**
  * gtk_list_store_new:
  * @n_columns: number of columns in the list store
@@ -281,11 +445,13 @@ gtk_list_store_new (gint n_columns,
     {
       GType type = va_arg (args, GType);
       if (! _gtk_tree_data_list_check_type (type))
-	{
-	  g_warning ("%s: Invalid type %s\n", G_STRLOC, g_type_name (type));
-	  g_object_unref (retval);
-	  return NULL;
-	}
+        {
+          g_warning ("%s: Invalid type %s\n", G_STRLOC, g_type_name (type));
+          g_object_unref (retval);
+          va_end (args);
+
+          return NULL;
+        }
 
       gtk_list_store_set_column_type (retval, i, type);
     }
@@ -481,7 +647,10 @@ gtk_list_store_get_iter (GtkTreeModel *tree_model,
   i = gtk_tree_path_get_indices (path)[0];
 
   if (i >= g_sequence_get_length (seq))
-    return FALSE;
+    {
+      iter->stamp = 0;
+      return FALSE;
+    }
 
   iter->stamp = priv->stamp;
   iter->user_data = g_sequence_get_iter_at_pos (seq, i);
@@ -520,7 +689,7 @@ gtk_list_store_get_value (GtkTreeModel *tree_model,
   gint tmp_column = column;
 
   g_return_if_fail (column < priv->n_columns);
-  g_return_if_fail (VALID_ITER (iter, list_store));
+  g_return_if_fail (iter_is_valid (iter, list_store));
 		    
   list = g_sequence_get (iter->user_data);
 
@@ -773,10 +942,10 @@ gtk_list_store_set_value (GtkListStore *list_store,
 			  gint          column,
 			  GValue       *value)
 {
-  GtkListStorePrivate *priv = list_store->priv;
+  GtkListStorePrivate *priv;
 
   g_return_if_fail (GTK_IS_LIST_STORE (list_store));
-  g_return_if_fail (VALID_ITER (iter, list_store));
+  g_return_if_fail (iter_is_valid (iter, list_store));
   g_return_if_fail (G_IS_VALUE (value));
   priv = list_store->priv;
   g_return_if_fail (column >= 0 && column < priv->n_columns);
@@ -935,7 +1104,7 @@ gtk_list_store_set_valuesv (GtkListStore *list_store,
   gboolean maybe_need_sort = FALSE;
 
   g_return_if_fail (GTK_IS_LIST_STORE (list_store));
-  g_return_if_fail (VALID_ITER (iter, list_store));
+  g_return_if_fail (iter_is_valid (iter, list_store));
 
   priv = list_store->priv;
 
@@ -977,7 +1146,7 @@ gtk_list_store_set_valist (GtkListStore *list_store,
   gboolean maybe_need_sort = FALSE;
 
   g_return_if_fail (GTK_IS_LIST_STORE (list_store));
-  g_return_if_fail (VALID_ITER (iter, list_store));
+  g_return_if_fail (iter_is_valid (iter, list_store));
 
   priv = list_store->priv;
 
@@ -1047,7 +1216,7 @@ gtk_list_store_remove (GtkListStore *list_store,
   GSequenceIter *ptr, *next;
 
   g_return_val_if_fail (GTK_IS_LIST_STORE (list_store), FALSE);
-  g_return_val_if_fail (VALID_ITER (iter, list_store), FALSE);
+  g_return_val_if_fail (iter_is_valid (iter, list_store), FALSE);
 
   priv = list_store->priv;
 
@@ -1121,7 +1290,7 @@ gtk_list_store_insert (GtkListStore *list_store,
   iter->stamp = priv->stamp;
   iter->user_data = ptr;
 
-  g_assert (VALID_ITER (iter, list_store));
+  g_assert (iter_is_valid (iter, list_store));
 
   priv->length++;
   
@@ -1157,7 +1326,7 @@ gtk_list_store_insert_before (GtkListStore *list_store,
   priv = list_store->priv;
 
   if (sibling)
-    g_return_if_fail (VALID_ITER (sibling, list_store));
+    g_return_if_fail (iter_is_valid (sibling, list_store));
 
   if (!sibling)
     after = g_sequence_get_end_iter (priv->seq);
@@ -1193,7 +1362,7 @@ gtk_list_store_insert_after (GtkListStore *list_store,
   priv = list_store->priv;
 
   if (sibling)
-    g_return_if_fail (VALID_ITER (sibling, list_store));
+    g_return_if_fail (iter_is_valid (sibling, list_store));
 
   if (!sibling)
     after = g_sequence_get_begin_iter (priv->seq);
@@ -1304,20 +1473,10 @@ gboolean
 gtk_list_store_iter_is_valid (GtkListStore *list_store,
                               GtkTreeIter  *iter)
 {
-  GtkListStorePrivate *priv;
-
   g_return_val_if_fail (GTK_IS_LIST_STORE (list_store), FALSE);
   g_return_val_if_fail (iter != NULL, FALSE);
 
-  priv = list_store->priv;
-
-  if (!VALID_ITER (iter, list_store))
-    return FALSE;
-
-  if (g_sequence_iter_get_sequence (iter->user_data) != priv->seq)
-    return FALSE;
-
-  return TRUE;
+  return iter_is_valid (iter, list_store);
 }
 
 static gboolean real_gtk_list_store_row_draggable (GtkTreeDragSource *drag_source,
@@ -1650,8 +1809,8 @@ gtk_list_store_swap (GtkListStore *store,
 
   g_return_if_fail (GTK_IS_LIST_STORE (store));
   g_return_if_fail (!GTK_LIST_STORE_IS_SORTED (store));
-  g_return_if_fail (VALID_ITER (a, store));
-  g_return_if_fail (VALID_ITER (b, store));
+  g_return_if_fail (iter_is_valid (a, store));
+  g_return_if_fail (iter_is_valid (b, store));
 
   priv = store->priv;
 
@@ -1716,9 +1875,9 @@ gtk_list_store_move_before (GtkListStore *store,
   
   g_return_if_fail (GTK_IS_LIST_STORE (store));
   g_return_if_fail (!GTK_LIST_STORE_IS_SORTED (store));
-  g_return_if_fail (VALID_ITER (iter, store));
+  g_return_if_fail (iter_is_valid (iter, store));
   if (position)
-    g_return_if_fail (VALID_ITER (position, store));
+    g_return_if_fail (iter_is_valid (position, store));
 
   if (position)
     pos = g_sequence_iter_get_position (position->user_data);
@@ -1749,9 +1908,9 @@ gtk_list_store_move_after (GtkListStore *store,
   
   g_return_if_fail (GTK_IS_LIST_STORE (store));
   g_return_if_fail (!GTK_LIST_STORE_IS_SORTED (store));
-  g_return_if_fail (VALID_ITER (iter, store));
+  g_return_if_fail (iter_is_valid (iter, store));
   if (position)
-    g_return_if_fail (VALID_ITER (position, store));
+    g_return_if_fail (iter_is_valid (position, store));
 
   if (position)
     pos = g_sequence_iter_get_position (position->user_data) + 1;
@@ -1799,8 +1958,8 @@ gtk_list_store_compare_func (GSequenceIter *a,
   iter_b.stamp = priv->stamp;
   iter_b.user_data = (gpointer)b;
 
-  g_assert (VALID_ITER (&iter_a, list_store));
-  g_assert (VALID_ITER (&iter_b, list_store));
+  g_assert (iter_is_valid (&iter_a, list_store));
+  g_assert (iter_is_valid (&iter_b, list_store));
 
   retval = (* func) (GTK_TREE_MODEL (list_store), &iter_a, &iter_b, data);
 
@@ -2074,7 +2233,7 @@ gtk_list_store_insert_with_values (GtkListStore *list_store,
   iter->stamp = priv->stamp;
   iter->user_data = ptr;
 
-  g_assert (VALID_ITER (iter, list_store));
+  g_assert (iter_is_valid (iter, list_store));
 
   priv->length++;
 
@@ -2154,7 +2313,7 @@ gtk_list_store_insert_with_valuesv (GtkListStore *list_store,
   iter->stamp = priv->stamp;
   iter->user_data = ptr;
 
-  g_assert (VALID_ITER (iter, list_store));
+  g_assert (iter_is_valid (iter, list_store));
 
   priv->length++;  
 
